<?php


class Base_Doctrine_Table extends Doctrine_Table
{


    public function hasField($field, $check_translation = true)
    {
        if(parent::hasField($field)){
            return true;
        }elseif($check_translation && $this->hasRelation('Translations')){
            $tableTr =   $this->getRelation('Translations')->getTable();
            if($tableTr->hasField($field)){
                return true;
            }
        }

        return false;
    }


    /**
     * Find records basing on a field.
     *
     * @param string $column            field for the WHERE clause
     * @param string $value             prepared statement parameter
     * @param integer $id_language      id of language
     * @param int $hydrationMode        Doctrine_Core::HYDRATE_ARRAY or Doctrine_Core::HYDRATE_RECORD
     * @return Doctrine_Collection|array
     */
    public function findBy($fieldName, $value, $id_language = null, $hydrationMode = null)
    {
        $query = $this->createQuery('dctrn_find')
            ->where($this->buildFindByWhere($fieldName), (array) $value);

        if($this->hasRelation('Translations')){
            $id_language = null === $id_language ? Base_I18n::getLangId() : $id_language;
            $query->leftJoin('dctrn_find.Translations t WITH t.id_language = ?', $id_language);
        }

        return $query->execute(array(), $hydrationMode);
    }

    /**
     * Finds the first record that satisfy the clause.
     *
     * @param string $column            field for the WHERE clause
     * @param string $value             prepared statement parameter
     * @param integer $id_language      id of language
     * @param int $hydrationMode        Doctrine_Core::HYDRATE_ARRAY or Doctrine_Core::HYDRATE_RECORD
     * @return Doctrine_Record
     */
    public function findOneBy($fieldName, $value, $id_language = null, $hydrationMode = null)
    {

        $query = $this->createQuery('dctrn_find')
            ->where($this->buildFindByWhere($fieldName), (array) $value)
            ->limit(1);

        if($this->hasRelation('Translations')){
            $id_language = null === $id_language ? Base_I18n::getLangId() : $id_language;
            $query->leftJoin('dctrn_find.Translations t WITH t.id_language = ?', $id_language);
        }

        return $query->fetchOne(array(), $hydrationMode);
    }


    /**
     * Adds support for magic finders.
     *
     * This method add support for calling methods not defined in code, such as:
     * findByColumnName, findByRelationAlias
     * findById, findByContactId, etc.
     *
     * @return the result of the finder
     */
    public function __call($method, $arguments)
    {
        $lcMethod = strtolower($method);

        if (substr($lcMethod, 0, 6) == 'findby') {
            $by = substr($method, 6, strlen($method));
            $method = 'findBy';
        } else if (substr($lcMethod, 0, 9) == 'findoneby') {
            $by = substr($method, 9, strlen($method));
            $method = 'findOneBy';
        }

        if (isset($by)) {
            if ( ! isset($arguments[0])) {
                throw new Doctrine_Table_Exception('You must specify the value to ' . $method);
            }

            $fieldName = $this->_resolveFindByFieldName($by);
            $count = count(explode('Or', $by)) + (count(explode('And', $by)) - 1);
            $id_language = null;
            $hydrationMode = null;
            $count_arguments = count($arguments);

            if($count_arguments > $count+1){
                $hydrationMode = end($arguments);
                unset($arguments[count($arguments) - 1]);
                $id_language = end($arguments);
                unset($arguments[count($arguments) - 1]);
            }elseif($count_arguments > $count){
                $id_language = end($arguments);
                unset($arguments[count($arguments) - 1]);
            }

            if ($this->hasField($fieldName)) {
                return $this->$method($fieldName, $arguments[0], $id_language, $hydrationMode);
            } else if ($this->hasRelation($by)) {
                $relation = $this->getRelation($by);

                if ($relation['type'] === Doctrine_Relation::MANY) {
                    throw new Doctrine_Table_Exception('Cannot findBy many relationship.');
                }

                return $this->$method($relation['local'], $arguments[0], $id_language, $hydrationMode);
            } else {
                return $this->$method($by, $arguments, $id_language, $hydrationMode);
            }
        }

        // Forward the method on to the record instance and see if it has anything or one of its behaviors
        try {
            return call_user_func_array(array($this->getRecordInstance(), $method . 'TableProxy'), $arguments);
        } catch (Doctrine_Record_UnknownPropertyException $e) {}

        throw new Doctrine_Table_Exception(sprintf('Unknown method %s::%s', get_class($this), $method));
    }

}