<?php

abstract class Base_Form extends Zend_Form
{
    /**
     * Class constants
     */
    const DISPOSITION_HORIZONTAL = 'horizontal';
    const DISPOSITION_INLINE     = 'inline';
    const DISPOSITION_SEARCH     = 'search';

    protected $_prefixesInitialized = false;

    protected $_belong_to = '';

    protected $_tlabel = '';

    /**
     * @var Base_Doctrine_Record
     */
    protected $_model = null;

    protected $_merge_model = true;

    protected $_filter = false;

    private $_is_filtred = false;

    protected $_disposition = null;

    /**
     * @var null
     */
    protected $_idQuery = null;

    /**
     * @var Zend_Session_Namespace
     */
    protected  $_session;

    /**
     * @param $model Doctrine_Record
     */
    protected function setModel($model)
    {
        $this->_model = $model;
    }

    /**
     * @param $filter boolean true|false
     */
    public function setFilter($filter)
    {
        $this->_filter = $filter;
    }


    /**
     * Override the base form constructor.
     *
     * @param null $options
     */
    public function __construct($options = null)
    {
        $this->_initializePrefixes();

        if(empty($this->_belong_to)){
            $this->_belong_to = get_called_class();
        }
        $this->setElementsBelongTo($this->_belong_to);

        if(empty($this->_tlabel)){
            $this->_tlabel = strtolower(get_called_class()).'_';
        }

        $this->setDecorators(array(
            'FormElements',
            array('Form', array('role' => 'form'))
        ));

        parent::__construct($options);

        if ( true === $this->_filter ) {
            $this->_checkSessionFilter();
        }
    }



    protected function _checkSessionFilter()
    {
        $request = Zend_Controller_Front::getInstance()->getRequest();

        $valueDefault = array_filter($this->getValues($this->_belong_to));
        $data = (array) $request->getParam($this->_belong_to, array());


        if ( null === $this->_session ) {
            $this->_session = new Zend_Session_Namespace($this->_belong_to);
        }

        if(!$this->_session->__isset('filter')){
            $this->_session->__set('filter', json_encode(array(
                'data' => $this->getValues($this->_belong_to),
                'default' => $this->getValues($this->_belong_to),
                'id_query' => null,
            )));
        }



        $filterData = json_decode($this->_session->filter, true);


        if($this->_idQuery != null){
            $filterData['id_query'] = $this->_idQuery;
            $filterData['default'] = $valueDefault;
            $data = $valueDefault;
        }

        $clear = $request->getParam('clear', 0);
        if($clear) {
            $filterData['data'] = $this->getValues($this->_belong_to);
            $filterData['default'] = $this->getValues($this->_belong_to);
            $filterData['id_query'] = null;
        }


        if(!empty($data)){
            $filterData['data'] = $data;
        }



        $this->_idQuery = $filterData['id_query'];
        $this->_session->__set('filter', json_encode($filterData));


        $this->populate($filterData['data']);


        if($filterData['data'] != $filterData['default']){
            $this->_is_filtred = true;
        }
    }


    /**
     * @return bool
     */
    public function hasIdQuery()
    {
        return (bool) $this->_idQuery;
    }

    /**
     * @return null|int
     */
    public function getIdQuery()
    {
        return $this->_idQuery;
    }

    /**
     * @return bool
     */
    public function isFiltred()
    {
        return $this->_is_filtred;
    }

    /**
     * @return bool
     */
    public function isFilter()
    {
        return (bool) $this->_filter;
    }

    protected function _initializePrefixes()
    {
        if (!$this->_prefixesInitialized){
            $this->addPrefixPath('Base_Form_Element', 'Base/Form/Element', 'element');
            $this->addElementPrefixPath('Base_Form_Decorator', 'Base/Form/Decorator', 'decorator');
            $this->addDisplayGroupPrefixPath('Base_Form_Decorator', 'Base/Form/Decorator' );
            $this->addElementPrefixPath('Base_Validate', 'Base/Validate/', 'validate');
            $this->setDefaultDisplayGroupClass('Base_Form_DisplayGroup');

            $this->_prefixesInitialized = true;
        }
    }

    /**
     * @param string $disposition
     */
    public function setDisposition($disposition)
    {
        if (
        in_array(
            $disposition,
            array(
                self::DISPOSITION_HORIZONTAL,
                self::DISPOSITION_INLINE,
                self::DISPOSITION_SEARCH
            )
        )
        ) {
            $this->_addClassNames('form-' . $disposition);
            $this->_disposition = $disposition;
        }
    }

    /**
     * Adds a class name
     *
     * @param string $classNames
     */
    protected function _addClassNames($classNames)
    {
        $classes = $this->_getClassNames();

        foreach ((array) $classNames as $className) {
            $classes[] = $className;
        }

        $this->setAttrib('class', trim(implode(' ', array_unique($classes))));
    }

    /**
     * Removes a class name
     *
     * @param string $classNames
     */
    protected function _removeClassNames($classNames)
    {
        $this->setAttrib('class', trim(implode(' ', array_diff($this->_getClassNames(), (array) $classNames))));
    }

    /**
     * Extract the class names from a Zend_Form_Element if given or from the
     * base form
     *
     * @param Zend_Form_Element $element
     * @return array
     */
    protected function _getClassNames(Zend_Form_Element $element = null)
    {
        if (null !== $element) {
            return explode(' ', $element->getAttrib('class'));
        }

        return explode(' ', $this->getAttrib('class'));
    }

    /**
     * Render form
     *
     * @param  Zend_View_Interface $view
     * @return string
     */
    public function render(Zend_View_Interface $view = null) {

        /**
         * Getting elements.
         */
        $elements = $this->getElements();

        foreach ($elements as $eachElement) {
            /**
             * Removing label from buttons before render.
             */
            if ($eachElement instanceof Zend_Form_Element_Submit) {
                $eachElement->removeDecorator('Label');
            }

            /**
             * No decorators for hidden elements
             */
            if( $eachElement instanceof Zend_Form_Element_Hidden ) {
                $eachElement->clearDecorators()->addDecorator('ViewHelper');
            }

            /**
             * No decorators for hash elements
             */
            if( $eachElement instanceof Zend_Form_Element_Hash ) {
                $eachElement->clearDecorators()->addDecorator('ViewHelper');
            }


            if($this->_disposition == self::DISPOSITION_HORIZONTAL && false !== ($labelDecorator = $eachElement->getDecorator('Label'))){
                $size = $eachElement->getAttrib('label-size');
                if(empty($size)){ $size = 2; }

                $class = $labelDecorator->getOption('class').' col-md-'.$size;
                $labelDecorator->setOption('class', $class);
            }
        }

        /**
         * Rendering.
         */
        return parent::render($view);
    }

    public function addHtmlTag($elements, $options = array(), $decorator_name = null)
    {
        $elements = array_values((array)$elements);

        $options_open = array_merge($options, array('openOnly' => true));
        $options_close = array_merge($options, array('closeOnly' => true));

        $decorator_name = empty($decorator_name) ? count($elements[0]->getDecorators()) : $decorator_name;
        $dec_open_name = 'open_'.$decorator_name;
        $dec_close_name = 'close_'.$decorator_name;

        $elements[0]->addDecorator(array($dec_open_name => 'HtmlTag'), $options_open);
        $elements[count($elements)-1]->addDecorator(array($dec_close_name => 'HtmlTag'), $options_close);

    }

    public function isValid($data)
    {
        return parent::isValid($data) && $this->postIsValid($data);
    }

    protected function postIsValid($data)
    {
        if($this->_merge_model && null !== $this->_model && $this->_model instanceof Base_Doctrine_Record){
            $this->_model->setFields($this->getValues($this->_belong_to));
        }

        return true;
    }


    public function getValuesQuery($suppressArrayNotation = false)
    {

        $values = parent::getValues($suppressArrayNotation);

        if(!$this->isValid($values)){
            $this->populate($this->_listData['filter']);
            $values = $this->_listData['filter'];
        }


        return $values;
    }


}
